#!/bin/bash

###
 # Copyright 2017 Information and Computational Sciences,
 # The James Hutton Institute.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
 #     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
###

#### Written by: Sebastian Raubach - sebastian.raubach@hutton.ac.uk

set -e

# Parameters: program name, potential program path
function getProgramPath {
	if [ -z "$2" ]; then
		path=`type -P $1`

		if [ -z "$path" ]; then
			echo "Command $1 not found"
			exit
		else
			echo $path
		fi
	else
		echo $2
	fi
}

# Parameters: parameter name, user prompt text, secure input (password), is parameter allowed to be empty?, potential parameter value?
function getParameter {
	# If the parameter isn't already set
	if [ -z "$5" ]; then
		# Check if it's a password
		if [ "$3" = true ]; then
			read -p "$2`echo $'\n> '`" -s parameter
		else
			read -p "$2`echo $'\n> '`" parameter
		fi

		# Check the user input is empty
		if [ -z "$parameter" ]; then
			# If it's not allowed to be empty, fail
			if [ "$4" = false ]; then
				>&2 echo "Parameter $1 not found"
				return 1
			else
				return 0
			fi
		# If the user input is ok, return it
		else
			echo $parameter
		fi
	# If we already have the parameter, then just return it again
	else
		echo $5
	fi
}

VERSION='v3.6.0'
VERSION_MYSQL='3_6_0'

source ./config.cfg

ant_path=$(getProgramPath "ant" $ant_path)
echo "Ant path set as: $ant_path"
git_path=$(getProgramPath "git" $git_path)
echo "Git path set as: $git_path"
mysql_path=$(getProgramPath "mysql" $mysql_path)
echo "MySQL path set as: $mysql_path"

echo ""
echo "########## Downloading Germinate from SVN ##########"

if [ ! -d "germinate" ] ; then
	$git_path clone https://github.com/germinateplatform/germinate.git --branch $VERSION germinate
fi

if [ $? -ne 0 ]; then
    echo "git checkout failed, terminating..."
    exit
fi

echo ""
echo "########## Downloading Germinate Gatekeeper from SVN ##########"

if [ ! -d "gatekeeper" ] ; then
$git_path clone https://github.com/germinateplatform/germinate-gatekeeper.git --branch $VERSION gatekeeper
fi

# Read a bunch of parameters
project_name=$(getParameter "Project name" "Please enter project name; lower case and no spaces ideally (e.g. 'barley-wild-relatives')" false false $project_name)

# Convert to lower case
#project_name=echo "$project_name" | tr '[:upper:]' '[:lower:]'

# Replace spaces with dashes
project_name="${project_name// /-}"

database_port="3306"
database_server=$(getParameter "Database server" "Please enter database server location" false false $database_server)
database_port=$(getParameter "Database server port" "Please enter database server port (leave empty for default port)" false true $database_port)
database_username=$(getParameter "Database username" "Please enter database username" false false $database_username)
database_password=$(getParameter "Database password" "Please enter database password" true true $database_password)
database_useport=""

if [ -z "$database_password" ]; then
	database_password=""
fi

if [ "$database_port" -eq "3306" ]; then
	database_useport="false"
else
	database_useport="true"
fi

tomcat_url=$(getParameter "Tomcat URL" "Please enter the Tomcat base url (e.g. http://localhost:8080/)" false false $tomcat_url)
tomcat_username=$(getParameter "Tomcat username" "Please enter the Tomcat username" false false $tomcat_username)
tomcat_password=$(getParameter "Tomcat password" "Please enter the Tomcat password" true false $tomcat_password)

email_address=$(getParameter "Gatekeeper email address" "Please enter the Gatekeeper email address" false true $email_address)
email_server=$(getParameter "Gatekeeper email server" "Please enter the Gatekeeper email server" false true $email_server)
email_port=$(getParameter "Gatekeeper email server port" "Please enter the Gatekeeper email server port (leave empty for default port)" false true $email_port)
email_username=$(getParameter "Gatekeeper username" "Please enter the Gatekeeper email username" false true $email_username)
email_password=$(getParameter "Gatekeeper password" "Please enter the Gatekeeper email password" true true $email_password)

# Remove potentially existing old folders with this project name
rm -rf germinate/instance-stuff/$project_name
# Then rename the template folder
cp -r germinate/instance-stuff/template germinate/instance-stuff/$project_name

# Generate the Germinate username and password
germinate_username="gm8_user"
germinate_password=`head /dev/urandom | tr -dc A-Za-z0-9 | head -c 32 ; echo ''`

set +e
echo ""
echo "########## Creating Germinate database user ##########"
$mysql_path -u $database_username --password=$database_password -h $database_server -P $database_port -e "CREATE USER '$germinate_username'@'%' IDENTIFIED BY '$germinate_password'; FLUSH PRIVILEGES;"

# User seems to exist, so remove it and then recreate it
if [ $? -ne 0 ]; then
    read -n1 -p "User $germinate_username seems to exist already. For this script to continue, it has to be removed and recreated. Do you want to proceed? [y/n]`echo $'\n> '`" doit 
	case $doit in  
		y|Y) ;; 
		n|N) exit ;; 
		*) exit ;; 
	esac
	
	$mysql_path -u $database_username --password=$database_password -h $database_server -P $database_port -e "DROP USER '$germinate_username'@'%'; FLUSH PRIVILEGES; CREATE USER '$germinate_username'@'%' IDENTIFIED BY '$germinate_password'; FLUSH PRIVILEGES;"
fi

echo ""
echo "########## Creating Germinate database ##########"
$mysql_path -u $database_username --password=$database_password -h $database_server -P $database_port -e "CREATE DATABASE germinate_$VERSION_MYSQL; GRANT ALL PRIVILEGES ON germinate_$VERSION_MYSQL.* TO '$germinate_username'@'%' IDENTIFIED BY '$germinate_password'; FLUSH PRIVILEGES;"

if [ $? -ne 0 ]; then
    read -n1 -p "Database germinate_$VERSION_MYSQL seems to exist already. For this script to continue, it has to be removed and recreated. Do you want to proceed? [y/n]`echo $'\n> '`" doit 
	case $doit in  
		y|Y) ;; 
		n|N) exit ;; 
		*) exit ;; 
	esac
	
	$mysql_path -u $database_username --password=$database_password -h $database_server -P $database_port -e "DROP DATABASE germinate_$VERSION_MYSQL; CREATE DATABASE germinate_$VERSION_MYSQL; GRANT ALL PRIVILEGES ON germinate_$VERSION_MYSQL.* TO '$germinate_username'@'%' IDENTIFIED BY '$germinate_password'; FLUSH PRIVILEGES;"
fi

set -e

echo ""
echo "########## Executing Germinate database creation script ##########"
$mysql_path -u $database_username --password=$database_password -h $database_server -P $database_port germinate_$VERSION_MYSQL < germinate/database/germinate_template.sql

set +e

echo ""
echo "########## Creating Germinate Gatekeeper database ##########"
$mysql_path -u $database_username --password=$database_password -h $database_server -P $database_port -e "CREATE DATABASE germinate_gatekeeper_$VERSION_MYSQL; GRANT ALL PRIVILEGES ON germinate_gatekeeper_$VERSION_MYSQL.* TO '$germinate_username'@'%' IDENTIFIED BY '$germinate_password'; FLUSH PRIVILEGES;"

if [ $? -ne 0 ]; then
    read -n1 -p "Database germinate_gatekeeper_$VERSION_MYSQL seems to exist already. For this script to continue, it has to be removed and recreated. Do you want to proceed? [y/n]`echo $'\n> '`" doit 
	case $doit in  
		y|Y) ;; 
		n|N) exit ;; 
		*) exit ;; 
	esac
	
	$mysql_path -u $database_username --password=$database_password -h $database_server -P $database_port -e "DROP DATABASE germinate_gatekeeper_$VERSION_MYSQL; CREATE DATABASE germinate_gatekeeper_$VERSION_MYSQL; GRANT ALL PRIVILEGES ON germinate_gatekeeper_$VERSION_MYSQL.* TO '$germinate_username'@'%' IDENTIFIED BY '$germinate_password'; FLUSH PRIVILEGES;"
fi

set -e

current_directory=$(pwd)

echo ""
echo "########## Executing Germinate Gatekeeper database creation script ##########"
$mysql_path -u $database_username --password=$database_password -h $database_server -P $database_port germinate_gatekeeper_$VERSION_MYSQL < gatekeeper/database/germinate_gatekeeper.sql

echo ""
echo "########## Inserting the Germinate system into Gatekeeper ##########"
$mysql_path -u $database_username --password=$database_password -h $database_server -P $database_port -e "USE germinate_gatekeeper_$VERSION_MYSQL; INSERT INTO database_systems (system_name, server_name, description) VALUES ('germinate_$VERSION_MYSQL', '$database_server', '$project_name');"

# Prepare the build of Germinate Gatekeeper
echo "########## Building Germinate Gatekeeper ##########"
tomcat_url=$(echo "$tomcat_url"|sed 's/\/$//g')

cp gatekeeper/build.properties gatekeeper/build-dev.properties
cp gatekeeper/config.properties gatekeeper/config-dev.properties
sed -i -e "s|project\.name=.*|project\.name=$project_name-gatekeeper|g" gatekeeper/build-dev.properties
sed -i -e "s|tomcat\.manager\.url=.*|tomcat\.manager\.url=$tomcat_url/manager/text|g" gatekeeper/build-dev.properties
sed -i -e "s|tomcat\.manager\.username=.*|tomcat\.manager\.username=$tomcat_username|g" gatekeeper/build-dev.properties
sed -i -e "s|tomcat\.manager\.password=.*|tomcat\.manager\.password=$tomcat_password|g" gatekeeper/build-dev.properties

sed -i -e "s|email\.address=.*|email\.address=$email_address|g" gatekeeper/config-dev.properties
sed -i -e "s|email\.user=.*|email\.user=$email_username|g" gatekeeper/config-dev.properties
sed -i -e "s|email\.password=.*|email\.password=$email_password|g" gatekeeper/config-dev.properties
sed -i -e "s|email\.server=.*|email\.server=$email_server|g" gatekeeper/config-dev.properties
sed -i -e "s|email\.port=.*|email\.port=$email_port|g" gatekeeper/config-dev.properties

sed -i -e "s|database\.server=.*|database\.server=$database_server|g" gatekeeper/config-dev.properties
sed -i -e "s|database\.username=.*|database\.username=$germinate_username|g" gatekeeper/config-dev.properties
sed -i -e "s|database\.password=.*|database\.password=$germinate_password|g" gatekeeper/config-dev.properties
sed -i -e "s|database\.name=.*|database\.name=germinate_gatekeeper_$VERSION_MYSQL|g" gatekeeper/config-dev.properties
sed -i -e "s|database\.port=.*|database\.port=$database_port|g" gatekeeper/config-dev.properties
sed -i -e "s|database\.useport=.*|database\.useport=$database_useport|g" gatekeeper/config-dev.properties

$ant_path -Dbasedir="$current_directory/gatekeeper" -f gatekeeper/build.xml

# Prepare the build of Germinate
echo "########## Building Germinate ##########"

cp germinate/build.properties germinate/build-dev.properties
sed -i -e "s|project\.name=.*|project\.name=$project_name|g" germinate/build-dev.properties
sed -i -e "s|instance\.files=.*|instance\.files=instance-stuff/$project_name|g" germinate/build-dev.properties
sed -i -e "s|tomcat\.manager\.url=.*|tomcat\.manager\.url=$tomcat_url/manager/text|g" germinate/build-dev.properties
sed -i -e "s|tomcat\.manager\.username=.*|tomcat\.manager\.username=$tomcat_username|g" germinate/build-dev.properties
sed -i -e "s|tomcat\.manager\.password=.*|tomcat\.manager\.password=$tomcat_password|g" germinate/build-dev.properties

sed -i -e "s|Germinate\.Database\.Username=.*|Germinate\.Database\.Username=$germinate_username|g" germinate/instance-stuff/$project_name/config.properties
sed -i -e "s|Germinate\.Database\.Password=.*|Germinate\.Database\.Password=$germinate_password|g" germinate/instance-stuff/$project_name/config.properties
sed -i -e "s|Germinate\.Database\.Server=.*|Germinate\.Database\.Server=$database_server|g" germinate/instance-stuff/$project_name/config.properties
sed -i -e "s|Germinate\.Database\.Name=.*|Germinate\.Database\.Name=germinate_$VERSION_MYSQL|g" germinate/instance-stuff/$project_name/config.properties
sed -i -e "s|Germinate\.Database\.Port=.*|Germinate\.Database\.Port=$database_port|g" germinate/instance-stuff/$project_name/config.properties
sed -i -e "s|Germinate\.Database\.UsePort=.*|Germinate\.Database\.UsePort=$database_useport|g" germinate/instance-stuff/$project_name/config.properties
sed -i -e "s|Gatekeeper\.Database\.Server=.*|Gatekeeper\.Database\.Server=$database_server|g" germinate/instance-stuff/$project_name/config.properties
sed -i -e "s|Gatekeeper\.Database\.Name=.*|Gatekeeper\.Database\.Name=germinate_gatekeeper_$VERSION_MYSQL|g" germinate/instance-stuff/$project_name/config.properties
sed -i -e "s|Gatekeeper\.Database\.Port=.*|Gatekeeper\.Database\.Port=$database_port|g" germinate/instance-stuff/$project_name/config.properties
sed -i -e "s|Gatekeeper\.Database\.UsePort=.*|Gatekeeper\.Database\.UsePort=$database_useport|g" germinate/instance-stuff/$project_name/config.properties
sed -i -e "s|Gatekeeper\.URL=.*|Gatekeeper\.URL=$tomcat_url/$project_name-gatekeeper|g" germinate/instance-stuff/$project_name/config.properties

$ant_path -Dbasedir="$current_directory/germinate" -f germinate/build.xml


# SUCCESS!
echo "Germinate and Germinate Gatekeeper have successfully been set up!"
echo "Visit $tomcat_url/$project_name-gatekeeper to set up a Gatekeeper admin account and other user accounts."
echo "Visit $tomcat_url/$project_name to log in using any of the above user accounts."